﻿-- ********************************************************************************
-- Halo Minimalistic
-- A minimalistic halo range display.
-- By: Shenton
--
-- HaloMini.lua
-- ********************************************************************************

--[[-------------------------------------------------------------------------------
    Globals to locals
--]]-------------------------------------------------------------------------------

local _G = _G;
local print = print;
local pairs = pairs;
local IsItemInRange = IsItemInRange
local CheckInteractDistance = CheckInteractDistance;
local UnitClass = UnitClass;
local GetTalentInfo = GetTalentInfo;
local UnitExists = UnitExists;
local UnitIsFriend = UnitIsFriend;
local SlashCmdList = SlashCmdList;
local GetLocale = GetLocale;
local GetScreenWidth = GetScreenWidth;
local GameTooltip = GameTooltip;
local InterfaceOptionsFrame_OpenToCategory = InterfaceOptionsFrame_OpenToCategory;

--[[-------------------------------------------------------------------------------
    Localization
--]]-------------------------------------------------------------------------------

local L = {};

-- à : \195\160    è : \195\168    ì : \195\172    ò : \195\178    ù : \195\185
-- á : \195\161    é : \195\169    í : \195\173    ó : \195\179    ú : \195\186
-- â : \195\162    ê : \195\170    î : \195\174    ô : \195\180    û : \195\187
-- ã : \195\163    ë : \195\171    ï : \195\175    õ : \195\181    ü : \195\188
-- ä : \195\164                    ñ : \195\177    ö : \195\182
-- æ : \195\166                                    ø : \195\184
-- ç : \195\167                                    œ : \197\147
-- Ä : \195\132    Ö : \195\150    Ü : \195\156    ß : \195\159

if ( GetLocale() == "frFR" ) then
    L["LOCKED"] = "Bloqu\195\169";
    L["UNLOCKED"] = "D\195\169bloqu\195\169";
    L["NOT_ON_PRIEST"] = "Pas sur un pr\195\170tre, tout \195\170 \195\169t\195\169 d\195\169sactiv\195\169. Vous devriez d\195\169sactiver Halo Minimalistic \195\160 l'\195\169cran de s\195\169lection des personnages.";
    L["OPTIONS_VERSION"] = "Version: %s";
    L["OPTIONS_LOCK_DESC"] = "Bloque ou d\195\169bloque Halo Minimalistic.";
    L["DATABASE_WIPED"] = "Base de donn\195\169es r\195\169initialis\195\169e avec ses valeurs par d\195\169faut.";
    L["SCALE"] = "Echelle";
    L["OPTIONS_SCALE_DESC"] = "D\195\169finir l'\195\169chelle (taille) de Halo Minimalistic.";
    L["REFRESH_RATE"] = "Taux de rafraichissement";
    L["OPTIONS_REFRESH_RATE_DESC"] = "D\195\169finir le taux de rafraichissement de Halo Minimalistic.";
    L["COMMAND_1"] = "Sans aucun argument cela bloque ou d\195\169bloque Halo Minimalistic.";
    L["COMMAND_2"] = "/hm config - Ouvre le panneau de configuration.";
    L["COMMAND_3"] = "/hm reset - Remet \195\160 z\195\169ro la position de Halo Minimalistic au centre de l'\195\169cran.";
    --L["COMMAND_4"] = "/hm wipedb - Purge la base de donnée de configuration.";
    L["RESET_POS"] = "Reset position";
    L["OPTIONS_RESET_POS_DESC"] = "Remet \195\160 z\195\169ro la position de Halo Minimalistic au centre de l'\195\169cran.";
    L["TOOLTIP_UNLOCKED_INFO"] = "Halo Minimalistic est d\195\169bloqu\195\169. La distance ne sera pas mise \195\160 jour tant que ce sera le cas.";
else
    L["LOCKED"] = "Locked";
    L["UNLOCKED"] = "Unlocked";
    L["NOT_ON_PRIEST"] = "Not on a priest, disabling everything. You should disable Halo Minimalistic by the characters selection screen.";
    L["OPTIONS_VERSION"] = "Version: %s";
    L["OPTIONS_LOCK_DESC"] = "Lock or unlock the Halo Minimalistic frame.";
    L["DATABASE_WIPED"] = "Database wiped and set to default.";
    L["SCALE"] = "Scale";
    L["OPTIONS_SCALE_DESC"] = "Set the scale of the Halo Minimalistic frame.";
    L["REFRESH_RATE"] = "Refresh rate";
    L["OPTIONS_REFRESH_RATE_DESC"] = "Set the refresh rate of Halo Minimalistic.";
    L["COMMAND_1"] = "With no argument it will toggle lock/unlock.";
    L["COMMAND_2"] = "/hm config - Open the configuration panel.";
    L["COMMAND_3"] = "/hm reset - Reset frame position to the center of the screen.";
    --L["COMMAND_4"] = "/hm wipedb - Wipe options database.";
    L["RESET_POS"] = "Reset position";
    L["OPTIONS_RESET_POS_DESC"] = "Reset frame position to the center of the screen.";
    L["TOOLTIP_UNLOCKED_INFO"] = "Halo Minimalistic frame is unlocked. It will not update the target range until it is locked.";
end

--[[-------------------------------------------------------------------------------
    Main object
--]]-------------------------------------------------------------------------------

local A = CreateFrame("Frame");
A.L = L;
_G["HaloMinimalisticGlobal"] = A;

--[[-------------------------------------------------------------------------------
    Variables
--]]-------------------------------------------------------------------------------

A.version = GetAddOnMetadata("HaloMini", "Version");

A.defaultDB =
{
    point = "CENTER",
    relativeTo = "UIParent",
    relativePoint = "CENTER",
    offX = 0,
    offY = 0,
    locked = 1,
    scale = 1,
    refreshRate = 0.5,
};

--[[-------------------------------------------------------------------------------
    Methods
--]]-------------------------------------------------------------------------------

-- Print
function A:Print(text)
    print("|cff33ff99HaloMini:|r "..text);
end

-- The player got Halo?
function A:GotHalo()
    local _, _, _, _, selected = GetTalentInfo(18);

    if ( selected ) then return 1; end

    return nil;
end

-- Range check
-- 30651 Dertrok's First Wand -- 15 yards -- Harm
-- 10645 gnomish death ray -- 20 yards -- Harm
-- 24268 tailoring net (netherweave) -- 25 yards -- Harm
-- 10716 Gnomish Shrink Ray -- 30 yards -- Harm
-- 18904 Zorbin's Ultra-Shrinker -- 35 yards -- Harm

-- 1251 Linen Bandage -- 15 yards -- Help
-- 21519 mistletoe -- 20 yards -- Help
-- 56247 Box of Crossbow Bolts -- 25 yards -- Help
-- 1180 Scroll of Stamina -- 30 yards -- Help

-- CheckInteractDistance("unit", 1) -- 1 = inspect, 4 = follow -- both = 28 yards -- help & harm
function A:CheckRange()
    local r15, r20, r25, r30;

    if ( UnitIsFriend("player", "target") ) then -- Friendly target
        r15 = IsItemInRange(1251, "target");
        r20 = IsItemInRange(21519, "target");
        r25 = IsItemInRange(56247, "target");
        r30 = IsItemInRange(1180, "target");
    else
        r15 = IsItemInRange(30651, "target");
        r20 = IsItemInRange(10645, "target");
        r25 = IsItemInRange(24268, "target");
        r30 = IsItemInRange(10716, "target");
    end

    -- Setting values to nil even if not in range and capable of using this item spell
    if ( r15 and r15 == 0 ) then r15 = nil; end
    if ( r20 and r20 == 0 ) then r20 = nil; end
    if ( r25 and r25 == 0 ) then r25 = nil; end
    if ( r30 and r30 == 0 ) then r30 = nil; end

    return r15, r20, r25, r30;
end

-- Set the main frame scale
function A:SetScale()
    A.frame:SetScale(A.db.scale);
end

-- Set the main frame position
function A:SetPos()
    A.frame:ClearAllPoints();
    A.frame:SetPoint(A.db.point, A.db.relativeTo, A.db.relativePoint, A.db.offX, A.db.offY);
end

-- Reset frame position
function A:ResetPos()
    A.db.point = "CENTER";
    A.db.relativeTo = "UIParent";
    A.db.relativePoint = "CENTER";
    A.db.offX = 0;
    A.db.offY = 0;

    A:SetPos();
end

-- Lock the frame
function A:Lock()
    -- Lock it
    A.frame:SetMovable(nil);
    A.frame:EnableMouse(nil);
    A.frame:SetScript("OnDragStart", nil);
    A.frame:SetScript("OnDragStop", nil);
    A.frame:SetScript("OnEnter", nil);
    A.frame:SetScript("OnLeave", nil);

    -- Enable OnUpdate script
    A:EnableUpdate();

    -- Tell the player the frame is lock
    A:Print(L["LOCKED"]);
end

-- Unlock the frame
function A:Unlock()
    -- Disable OnUpdate script
    A.frame:SetScript("OnUpdate", nil);

    -- Set movable
    A.frame:SetMovable(1);
    A.frame:EnableMouse(1);
    A.frame:RegisterForDrag("LeftButton");
    A.frame:SetScript("OnDragStart", A.frame.StartMoving);
    A.frame:SetScript("OnDragStop", function()
        A.frame:StopMovingOrSizing();

        local point, relativeTo, relativePoint, offX, offY = A.frame:GetPoint(1);

        A.db.point = point;
        A.db.relativeTo = relativeTo;
        A.db.relativePoint = relativePoint;
        A.db.offX = offX;
        A.db.offY = offY;
    end);
    A.frame:SetScript("OnEnter", function()
        A:DisplayUnlockedTooltip();
    end);
    A.frame:SetScript("OnLeave", function()
        GameTooltip:Hide();
    end);

    -- Display arrows and green dot
    A.frame.middleGray:Hide();
    A.frame.middleGreen:Hide();
    A.frame.middleYellow:Hide();
    A.frame.middleRed:Hide();
    A.frame.middleInfo:Show();
    A.frame.left:Show();
    A.frame.right:Show();

    -- Tell the player the frame is unlock
    A:Print(L["UNLOCKED"]);
end

-- Toggle lock/unlock
function A:ToggleLock()
    if ( A.frame:IsMovable() ) then
        A:Lock();
    else
        A:Unlock();
    end
end

-- Display a warning tooltip when hovering unlocked main frame
function A:DisplayUnlockedTooltip()
    if ( A.frame:GetRight() >= ( GetScreenWidth() / 2 ) ) then
        GameTooltip:SetOwner(A.frame, "ANCHOR_LEFT");
    else
        GameTooltip:SetOwner(A.frame, "ANCHOR_RIGHT");
    end

    GameTooltip:AddDoubleLine("|cffffffffHalo Minimalistic", "|cff33ff99v"..A.version);
    GameTooltip:AddLine(" ");
    GameTooltip:AddLine(L["TOOLTIP_UNLOCKED_INFO"], nil, nil, nil, 1);

    GameTooltip:Show();
end

--[[-------------------------------------------------------------------------------
    Events
--]]-------------------------------------------------------------------------------

A:RegisterEvent("ADDON_LOADED");
A:RegisterEvent("PLAYER_LOGIN");

A:SetScript("OnEvent", function(self, event, ...)
    local arg1 = ...;

    if ( event == "ADDON_LOADED" and arg1 == "HaloMini" ) then
        -- Setting default savedvar if empty
        if ( not _G["haloMiniDB"] ) then _G["haloMiniDB"] = {}; end

        for k,v in pairs(A.defaultDB) do
            if ( not _G["haloMiniDB"][k] ) then
                _G["haloMiniDB"][k] = v;
            end
        end

        -- Setting savedvar to local object
        A.db = _G["haloMiniDB"];
    elseif ( event == "PLAYER_LOGIN" ) then
        local _, _, classID = UnitClass("player");

        if ( classID == 5 ) then -- It's a priest
            -- Setting frame and layers objects to local object
            A.frame = _G["HaloMiniFrame"];
            A.frame.middleGray = _G["HaloMiniFrameMiddleGray"];
            A.frame.middleGreen = _G["HaloMiniFrameMiddleGreen"];
            A.frame.middleYellow = _G["HaloMiniFrameMiddleYellow"];
            A.frame.middleRed = _G["HaloMiniFrameMiddleRed"];
            A.frame.middleInfo = _G["HaloMiniFrameMiddleInfo"];
            A.frame.left = _G["HaloMiniFrameLeft"];
            A.frame.right = _G["HaloMiniFrameRight"];

            -- On update value var
            A.frame.lastUpdate = 0;

            -- Priest got Halo?
            if ( A:GotHalo() ) then A.frame:Show(); end

            -- Setting options
            if ( not A.db.locked ) then A:Unlock(); end
            A:SetScale();
            A:SetPos();

            -- Setting slash commands
            A:SetSlashCommands()

            -- Adding options panel to Blizzard interface options panel
            A:CreateOptionsPanel();

            -- Starting main OnUpdate script
            A:EnableUpdate();

            -- Registering events we only want to fire when everything is loaded
            A:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED");
        else -- Not a priest
            A:Print(L["NOT_ON_PRIEST"]);
            A:UnregisterAllEvents();
            A = nil;
        end
    elseif ( event == "ACTIVE_TALENT_GROUP_CHANGED" ) then
        -- Player switched specialisation, got Halo?
        if ( A:GotHalo() ) then
            A.frame:Show();
        else
            A.frame:Hide();
        end
    end
end);

--[[-------------------------------------------------------------------------------
    OnUpdate scripts
--]]-------------------------------------------------------------------------------

-- Main OnUpdate script
function A:EnableUpdate()
    A.frame:SetScript("OnUpdate", function(self, elapsed)
        self.lastUpdate = self.lastUpdate + elapsed;

        if ( self.lastUpdate > A.db.refreshRate ) then
            if ( UnitExists("target") ) then -- Got a target
                -- Getting range status
                local r15, r20, r25, r30 = A:CheckRange();

                -- Setting layers
                if ( r15 ) then
                    A.frame.left:Show();
                    A.frame.right:Hide();
                    A.frame.middleGreen:Hide();
                    A.frame.middleYellow:Hide();
                    A.frame.middleRed:Show();
                elseif ( not r15 and r20 ) then
                    A.frame.left:Show();
                    A.frame.right:Hide();
                    A.frame.middleGreen:Hide();
                    A.frame.middleYellow:Show();
                    A.frame.middleRed:Hide();
                elseif ( not r20 and r25 ) then
                    A.frame.left:Hide();
                    A.frame.right:Hide();
                    A.frame.middleGreen:Show();
                    A.frame.middleYellow:Hide();
                    A.frame.middleRed:Hide();
                elseif ( not r25 and r30 ) then
                    A.frame.left:Hide();
                    A.frame.right:Show();
                    A.frame.middleGreen:Hide();
                    A.frame.middleYellow:Show();
                    A.frame.middleRed:Hide();
                elseif ( not r30 ) then
                    A.frame.left:Hide();
                    A.frame.right:Show();
                    A.frame.middleGreen:Hide();
                    A.frame.middleYellow:Hide();
                    A.frame.middleRed:Show();
                end

                A.frame.middleGray:Hide();
            else -- No target
                A.frame.middleGray:Show();
                A.frame.middleGreen:Hide();
                A.frame.middleYellow:Hide();
                A.frame.middleRed:Hide();
                A.frame.left:Hide();
                A.frame.right:Hide();
            end

            A.frame.middleInfo:Hide();
            self.lastUpdate = 0;
        end
    end);
end

--[[-------------------------------------------------------------------------------
    Slash command
--]]-------------------------------------------------------------------------------

function A:SetSlashCommands()
    _G["SLASH_HALOMINI1"], _G["SLASH_HALOMINI2"] = "/hm", "/halomini";
    function SlashCmdList.HALOMINI(args)
        if ( args == "" ) then
            A:ToggleLock();
        elseif ( args == "wipedb" ) then
            for k,v in pairs(A.defaultDB) do
                A.db[k] = v;
            end

            A:Print(L["DATABASE_WIPED"]);
        elseif ( args == "config" ) then
            InterfaceOptionsFrame_OpenToCategory(A.optionsPanel);
        elseif ( args == "reset" ) then
            A:ResetPos();
        else
            A:Print(L["COMMAND_1"]);
            A:Print(L["COMMAND_2"]);
            A:Print(L["COMMAND_3"]);
            --A:Print(L["COMMAND_4"]);
        end
    end
end
